#include <stdlib.h>
#include "httphelper.h"
#include "cehttp.h"
#include "str.h"

// ----------------------------------------------------------------------------
// HELPER FUNCTIONS
// ----------------------------------------------------------------------------

int GetCurrentDirW(WCHAR* pwszDir, int nBufLen)
{
	DWORD dwLen;

	memset( pwszDir, 0, sizeof(WCHAR)*nBufLen );

	dwLen = GetModuleFileName( NULL, pwszDir, nBufLen );
	
	if( dwLen == 0 )
		return 0;

#if defined(WIN32) && !defined(_WIN32_WCE)
	// Remove drive letter.
	WCHAR* pwSrc = pwszDir;
	WCHAR* pwDest = pwSrc;
	pwSrc += 2;

	dwLen -=2;	// length of drive letter.
	
	do
	{
		*pwDest++ = *pwSrc;
	}while( *pwSrc++ != '\0' );
#endif
	
	dwLen--;

	while( pwszDir[dwLen] != '\0' && pwszDir[dwLen] != '\\' )
	{
		dwLen--;
		if( dwLen == 0 )
			break;
	}

	pwszDir[dwLen] = '\0';

	return dwLen;
}

// ------------------------------------------------------------------
// BASE64
// ------------------------------------------------------------------
static const char base64_charset[] ="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
static char B64(char c)
{
	if( c == '=' ) return 0;
	for( char ii = 0; base64_charset[ii] != c && base64_charset[ii] != '\0'; ii++ );
	return ii;
}

char* Base64Decode(char *pInOut)
{
    char *p = pInOut;

    int len = STRLEN(pInOut);
    for (int ii=0,jj=0; ii < len; ii+=4, jj+=3)
	{
        p[jj]   =(char)(  (B64(pInOut[ii])           << 2) | ((B64(pInOut[ii+1]) & 0x30) >> 4) );
        p[jj+1] =(char)( ((B64(pInOut[ii+1]) & 0x0f) << 4) | ((B64(pInOut[ii+2]) & 0x3c) >> 2) );
        p[jj+2] =(char)( ((B64(pInOut[ii+2]) & 0x03) << 6) |  (B64(pInOut[ii+3]) & 0x3f)       );
    }
    p[jj] = '\0';
    return (p);
}

char *Base64Encode(const char *pIn)
{
	char *p;
	char *pRet;
	int ii, jj;
	int len = STRLEN(pIn);

	pRet = p = (char*)MALLOC((len * 8) / 6 + 4);

	for (ii= 0; ii < len / 3; ++ii)
	{
		*p++ = base64_charset[*pIn >> 2];
		jj = ((*pIn & 0x03) << 4);
		pIn++;
		*p++ = base64_charset[ jj | (*pIn >> 4) ];
		jj = ((*pIn & 0x0F) << 2);
		pIn++;
		*p++ = base64_charset[ jj | (*pIn >> 6) ];
		*p++ = base64_charset[ *pIn++ & 0x3F ];
	}

	switch (len % 3) {
	case 2:
		*p++ = base64_charset[*pIn >> 2];
		jj =  ((*pIn & 0x03) << 4);
		pIn++;
		*p++ = base64_charset[jj | (*pIn >> 4)];
		*p++ = base64_charset[((*pIn++ & 0x0F) << 2)];
		break;
	case 1:
		*p++ = base64_charset[*pIn >> 2];
		*p++ = base64_charset[(*pIn++ & 0x03) << 4];
		break;
	}
	*p++ = '=';
	*p++ = '=';
	*p = 0;
	return pRet;
}


//-----------------------------------------------------------------------------
// '/' -> '\\'
void slash2bslash(char* str)
{
	if( str == NULL )
		return;

	for( ;*str;str++)
		if( *str == '/' ) *str = '\\';
}

//-----------------------------------------------------------------------------
// '/' <- '\\'
void bslash2slash(char* str)
{
	if( str == NULL )
		return;

	for( ;*str;str++)
		if( *str == '\\' ) *str = '/';
}

//-----------------------------------------------------------------------------
// char to wchar with malloc
WCHAR* malloc_char2wchar( const char* pszStr )
{
	WCHAR* pwsRet = NULL;
	int nRet = 0;

	nRet = MultiByteToWideChar(CP_ACP,0,pszStr,-1,NULL,0);

	if( nRet == 0 )
	{
		return NULL;
	}

	pwsRet = (WCHAR*)MALLOC( nRet * sizeof(WCHAR) );

	if( pwsRet == NULL )
		return NULL;

	if( 0 == MultiByteToWideChar(CP_ACP,0,pszStr,-1,pwsRet,nRet) )
	{
		FREEIF( pwsRet );
		return NULL;
	}

	return pwsRet;

}

//-----------------------------------------------------------------------------
// FindFirstFile MBCS
HANDLE FindFirstFileMBCS( const char* lpFileName, LPWIN32_FIND_DATA lpFindFileData )
{
	WCHAR wszFile[MAX_PATH] = {0};

	if( 0 == MultiByteToWideChar(CP_ACP,0,lpFileName,-1, wszFile,MAX_PATH) )
	{
		return INVALID_HANDLE_VALUE;
	}

	return FindFirstFile( wszFile, lpFindFileData );
}

//-----------------------------------------------------------------------------
// GetFileAttributesMBCS MBCS
DWORD GetFileAttributesMBCS( const char* lpFileName )
{
	WCHAR wszFile[MAX_PATH] = {0};

	if( 0 == MultiByteToWideChar(CP_ACP,0,lpFileName,-1, wszFile,MAX_PATH) )
	{
		return 0xFFFFFFFF;
	}

	return GetFileAttributes( wszFile );
}